/* comparison.cpp */

#include <iostream>
#include <string>
#include <vector>
#include <fstream>
using namespace std;

#include "comparison.h"

comparison::comparison(){
    /* intentionally left blank */
}

void comparison::setStudentName(string name){
    student.setName(name);
}

void comparison::setSolutionName(string name){
    solution.setName(name);
}

programio comparison::getStudentProg(){
    return student;
}

programio comparison::getSolutionProg(){
    return solution;
}

void comparison::collectProgramOutput(char selection, string name, string inputFile, bool &loadStatus){
    if (selection == 't')
        prog = student;
    else if (selection == 'u')
        prog = solution;

    prog.setName(name);
    prog.collectOutput(inputFile);
    prog.readOutput();
    prog.removeCRs();
    prog.formatOutput();
    prog.printOutput();

    if (selection == 't')
        student = prog;
    else if (selection == 'u')
        solution = prog;
}

// compare the outputs
void comparison::compareOutput(programio prog1, programio prog2, vector<string> &missing, vector<string> &found){
    int prog1Lines = 0;
	int prog2Lines = 0;
	bool foundline;
	bool secondRun = !found.empty(); // we don't want to duplicate the matched lines

    for (prog1Lines; prog1Lines <= prog1.getLastIndex(); prog1Lines++){
		for (prog2Lines; prog2Lines <= prog2.getLastIndex(); prog2Lines++){
			if (prog1.getOutputVec()[prog1Lines] == prog2.getOutputVec()[prog2Lines]){ // line is the same
				foundline = true;
				break;
			}
		}
		if (!foundline)
			missing.push_back(prog1.getOutputVec()[prog1Lines]);
        else if(foundline && !secondRun)
            found.push_back(prog1.getOutputVec()[prog1Lines]);
		prog2Lines=0;
		foundline=false;
	}
}

// print the vector, for debugging (or more)
void comparison::printVector(vector<string> vec){
    for(vector<string>::iterator i = vec.begin(); i != vec.end(); i++){
		cout << *i;
	}
}

void comparison::printVecToFile(string ofilename, string label, vector<string> vec){
    ofstream ofs;
    ofs.open(ofilename.c_str(), ios::out | ios::app);
    if (!ofs.fail()){
        ofs << "*************************\n"; // delineate sections (25 *'s, if you were wondering)
        ofs << label << endl << endl;
        for(vector<string>::iterator i = vec.begin(); i != vec.end(); i++){
            ofs << *i;
        }
    }else
        cerr << "Problem writing output to file.\n";
    ofs.close();
    ofs.clear();
}

void comparison::compare(){
    compareOutput(student, solution, missingInStudent, linesMatched);
    compareOutput(solution, student, missingInSolution, linesMatched);
    cout << "Lines that were missing in student: \n";
    printVector(missingInStudent);
    cout << endl;
    cout << "Lines that were extra in student: \n";
    printVector(missingInSolution);
    cout << endl;
    cout << "Lines that were found in both: \n";
    printVector(linesMatched);
    cout << endl;
    writeCompToFile();
}

void comparison::clearCompVecs(){
    missingInStudent.clear();
    missingInSolution.clear();
    linesMatched.clear();
}

void comparison::writeCompToFile(){
    string outFileName = student.getName() + "-" + solution.getName() + ".comparison";
    printVecToFile(outFileName, "Lines missing from student:", missingInStudent);
    printVecToFile(outFileName, "Lines missing in solution:", missingInSolution);
    printVecToFile(outFileName, "Lines found in both student and solution:", linesMatched);
    cout << "Comparison written to " << outFileName << endl;
}
