/* main.cpp */

/*  OUTCHECK
    A program to compare the outputs of console programs.
    This has some of the functionality of redirects and diff,
    but is more extensible (and fun to write). It should be a nice
    simple tool for UF's CGS2421 class; I hope to add class
    specific functionality at some point.

    Written by Douglas Whitaker in June 2009

    Licensed under GPLv3 */

#include <iostream>
#include <fstream>
#include <sstream>
using namespace std;

#include "comparison.h"

void printMenu(void); // print menu options
int executeMenu(void);
void readInput(string inputFile, string &input, bool &loadedINF); // read the input file
void fileExists(bool &loadStatus, string tempInput); // basic check to see if a file exists

int main(int argc, char* argv[]){

    if (executeMenu()==0) // for now, everything relavent is in here (maybe do return executeMenu()?)
        return 0;
    else
        return 1; // something went wrong
}

void printMenu(void){ // Menu choices (obviously)
    cout << endl;
	cout << "OUTCHECK: Program Output Comparison (v3b)\n";
	cout << endl;
	cout << "Menu choices: \n";
	cout << endl;
	cout << " 0 - Quit\n";
	cout << " 1 - Load input file\n";
	cout << " 2 - Load student program\n";
	cout << " 3 - Load solution program\n";
	cout << " 4 - Compare programs\n";
	cout << " 5 - Reset\n";
	cout << " 6 - About\n";
	cout << endl;
}

int executeMenu(void){
    const char MENU_QUIT = '0';
	const char MENU_INPUT_FILE = '1';
	const char MENU_STUDENT_PROGRAM = '2';
	const char MENU_SOLUTION_PROGRAM = '3';
	const char MENU_COMPARE = '4';
	const char MENU_RESET = '5';
	const char MENU_ABOUT = '6';

	char choice;
	string tempInput, inputFile, input;
	bool loadedINF, loadedSTU, loadedSOL, compFail;
	loadedINF = loadedSTU = loadedSOL = compFail = false;
    comparison progcomp;

	do{
		printMenu();
		cout << "Enter selection: ";
		cin >> choice;
		switch(choice){
			case MENU_QUIT: // Quit
				return 0;
				break;

			case MENU_INPUT_FILE: // Input file
				cout << "Enter filename: ";
				cin >> inputFile;

				readInput(inputFile, input, loadedINF);

                if (loadedINF){
                    cout << "Input loaded: \n";
                    cout << input;
                }
                else
                    cerr << endl << "Error loading input file. Please check the spelling." << endl;

				break;

			case MENU_STUDENT_PROGRAM: // Student file
				cout << "Enter filename for student program: ";
				cin.ignore(); // ignore the \n from menu selection
				std::getline(cin, tempInput);
                cout << endl;

                fileExists(loadedSTU, tempInput);

                if (!loadedSTU){
                    cerr << "Error loading student program.\n";
                    cout << endl;
                    break;
                }
                else{
                    progcomp.collectProgramOutput('t', tempInput, inputFile, loadedSTU);
                    cout << endl << "Student program loaded." << endl;
                }

				break;

			case MENU_SOLUTION_PROGRAM: // Solution file
				cout << "Please enter the filename for the solution file: ";
				cin.ignore(); // ignore the \n from menu selection
				std::getline(cin, tempInput);
				cout << endl;

                fileExists(loadedSOL, tempInput);

                if (!loadedSOL){
                    cerr << "Error loading solution program.\n";
                    cout << endl;
                    break;
                }
                else{
                    progcomp.collectProgramOutput('u', tempInput, inputFile, loadedSOL);
                    cout << endl << "Solution program loaded." << endl;
                }

				break;

			case MENU_COMPARE: // Compare data. No error checking yet!
                cout << endl;
                if (!loadedINF){
                    cerr << "Error: Input file not loaded.\n";
                    compFail = true;
                }
                if (!loadedSTU){
                    cerr << "Error: Student program not loaded.\n";
                    compFail = true;
                }
                if (!loadedSOL){
                    cerr << "Error: Solution program not loaded.\n";
                    compFail = true;
                }
                if (compFail){
                    cerr << "Error(s) encountered. No comparison performed.\n" << endl;
                    compFail = false;
                    break;
                }

				cout << "Comparing program outputs. (A work in progress.)\n";
				cout << endl;

                progcomp.compare();

				break;

			case MENU_RESET: // Reset data
				loadedINF = loadedSTU = loadedSOL = false;
				progcomp.clearCompVecs();
                cout << "All inputs have been reset.\n";
				break;

			case MENU_ABOUT: // About
				cout << endl;
				cout << "OUTCHECK (v3b)\n";
				cout << endl;
				cout << "A program to compare the output of console applications.\n";
				cout << "Licensed under the GPLv3 (free to modify and distribute).\n";
				cout << "Written by Douglas Whitaker.";
				cout << endl;
				cout << "Information, source code, and updates available online at:\n";
				cout << "http://www.douglaswhitaker.com/outcheck/\n";
				cout << endl;
				break;

			default: // Invalid choice
				cout << "Error: Invalid input.";
				cout << endl;
				break;
		}
	}while(true);
}

void readInput(string fileName, string &input, bool &loadedINF){ // Read the input file, usually in .inp format
	std::ifstream data(fileName.c_str());
	if (!data.fail()){
        stringstream ss;
        ss << data.rdbuf();
        input = ss.str();
        loadedINF = true;
        data.close();
        data.clear();
	}
}

void fileExists(bool &loadStatus, string tempInput){
    ifstream fileTester;
    fileTester.open(tempInput.c_str());
    if (!fileTester.fail())
        loadStatus = true; // file exists at least...
    fileTester.close();
    fileTester.clear();
}
