/* programio.cpp */

#include <iostream>
#include <cstdlib> // exit()
#include <cstring> // strcmp() (replace this with == later)
#include <fstream>
using namespace std;

#include <errno.h> // error handling for execl()
#include <fcntl.h> // open(), close()
#include <unistd.h> // dup2(), fork(), getcwd()
#include <sys/param.h> // MAXPATHLEN
#include <sys/stat.h> // file permission flags
#include <sys/wait.h> // wait()

#include "programio.h" // header file containing the class def of program

programio::programio(): linesInOutput(0), WINE_PATH("/usr/bin/wine") {
    /* intentionally left blank */
}

programio& programio::operator=(programio& prog){
    fileName = prog.getName();
    output = prog.getOutputVec();
    WINE_PATH = prog.getWP();
    return *this;
}

void programio::setName(string name){
	fileName = name;
}

string programio::getName(){
    return fileName;
}

string programio::getWP(){
    return WINE_PATH;
}

vector<string> programio::getOutputVec(void){
	return output;
}

int programio::getLastIndex(void){
	//return linesInOutput;
	return output.size()-1;
}

void programio::collectOutput(string inputFile){
	char path[MAXPATHLEN];
	getcwd(path, MAXPATHLEN); // get CurrentWorkingDirectory
	string pathAndProg(path);
	pathAndProg = pathAndProg + "/" + fileName;
	string outputFile = fileName + ".output"; // make one string so we can use .c_str() later

	int pid; // to check fork()'s status

	int filedes1 = open(inputFile.c_str(), O_RDONLY); // open the inputFile (reading)
	int filedes2 = open(outputFile.c_str(), // open the file...
			O_WRONLY | O_CREAT | O_TRUNC, // write, create if it doesn't exist, ignore existing data
			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH); // set permissions -rw-r--r--

	if (filedes1 == -1){
		cerr << "Failed to open inputFile\n"; // don't set loadStatus to false because this doesn't relate to that program
	}
	if (filedes2 == -1){
		cerr << "Failed to open outputFile\n"; // don't set loadStatus to false because this doesn't relate to that program
	}

	if ((pid=fork()) == 0){ // child
	    if (!(fileName.find(".exe", fileName.length()-4) != string::npos))
            close(STDIN_FILENO); // only close STDIN_FILENO if wine will not be needed, good idea to do so though
		close(STDOUT_FILENO);

		dup2(filedes1, STDIN_FILENO); // duplicate the file descriptor for reading inputFile and attach it to stdin
		dup2(filedes2, STDOUT_FILENO); // duplicate the file descriptor for writing to fileName.output and attach it to stdout

		close(filedes1); // close the file descriptor for inputFile, not needed any longer
		close(filedes2); // close the file descriptor for fileName.output, not needed any longer

        if (fileName.find(".exe", fileName.length()-4) != string::npos) // if true, assume this is a Windows program
            execl(WINE_PATH.c_str(), "wine", pathAndProg.c_str(), (char *) 0);
        else // not a Windows program
            execl(pathAndProg.c_str(), fileName.c_str(), (char *) 0); // execute the program

		exit(0); // child exits
	}
	else if (pid > 0){ // parent
		wait(0); // wait for the child
		close(filedes1);
		close(filedes2);
	}
	else{
		perror("fork");
		exit(1);
	}
}

void programio::readOutput(void){
	string tempStr = "\n";
	string outName = fileName + ".output";
	output.clear();
	ifstream infile(outName.c_str());
	infile.clear();
	if(infile.is_open()){
		while(!infile.eof() && !infile.fail()){
			getline(infile,tempStr);
			if (!tempStr.empty()){
				output.push_back(tempStr + "\n");
				linesInOutput++;
				tempStr = "\n";
			}
			if (infile.eof() || infile.fail()){
				break;
			}
		}
	//cout << "Another important thing\n";
	infile.close();
	infile.clear();

	}
	else
		cout << "Something went wrong!\n";
}

// remove carriage returns from a document (i.e. convert Windows output to *nix)
void programio::removeCRs(void){
    for(vector<string>::iterator i = output.begin(); i != output.end(); i++){
        for(int j=0; j < (*i).length(); j++){
            if ((*i).at(j) == '\r'){
                (*i).erase(j);
                (*i).insert(j,"\n");
            }
        }
    }
}

// formatOutput removews all blank lines from the vector
// (as far as I know, blank lines are not part of the grading as of June 2009)
// maybe change this up to separate at :'s as well (maybe a separate function for that
void programio::formatOutput(void){
	int count=0;
	for(vector<string>::iterator i = output.begin(); i != output.end(); i++){
		if (strcmp(output[count].c_str(),"\n")==0){ // string does match a blank line
			output.erase(i);
			i--;
			linesInOutput--;
			count--;
		}
		count++;
	}
}

void programio::printOutput(void){ // print contents of vector to stdout, useful for debugging
	int count=0;
	for(vector<string>::iterator i = output.begin(); i != output.end(); i++){
		cout << output[count];
		count++;
	}
}


